# frozen_string_literal: true

require "net/http"
require "uri"
require "json"
require "date"
require "digest/md5"

require_relative "error"

module BBSIA
  # BBSIA API client class
  class Client
    BASE_URI = {
      prod: "https://gmtedi.bb.com.br",
      homol: "https://gmtedi.hm.bb.com.br"
    }.freeze

    attr_reader :user, :password, :env,
                :access_token, :refresh_token,
                :expires_in, :expires_at, :issued_at

    def initialize(user, password, env = :prod)
      @user = user
      @password = password
      @env = env
    end

    def base_uri
      BASE_URI[@env]
    end

    def auth
      uri = URI.parse("#{base_uri}/gmt-autorizador-api/autoriza")
      params = {
        "grant_type" => "password",
        "username" => @user,
        "password" => @password,
        "scope" => "sia:usuario"
      }

      res = Net::HTTP.post_form(uri, params)
      raise APIError, res if res.code != "200"

      data = JSON.parse(res.body)

      @access_token = data["access_token"]
      @refresh_token = data["refresh_token"]
      @expires_in = data["expires_in"]
      @issued_at = DateTime.now
      @expires_at = DateTime.rfc3339(data["data_expiracao"])

      data
    end

    def auth?
      !@access_token.nil?
    end

    def list_uploads
      uri = URI.parse("#{base_uri}/gmt-catalogo-api/listaUploads")
      headers = { "Authorization" => "Bearer #{@access_token}" }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.get(uri.path, headers)
      raise APIError, res if res.code != "200"

      data = JSON.parse(res.body)
      data["remessa"]
    end

    def get_upload_contract(contract)
      ftas = list_uploads
      ftas.select { |f| f["nome"].include?(contract) }.first
    end

    def list_downloads
      uri = URI.parse("#{base_uri}/gmt-sia-api/listaDownloads")
      headers = { "Authorization" => "Bearer #{@access_token}" }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.get(uri.path, headers)
      raise APIError, res if res.code != "200"

      data = JSON.parse(res.body)

      data["arquivos"]
    end

    def get_download_metadata(file_id, file_name)
      uri = URI.parse("#{base_uri}/gmt-sia-api/download/#{file_id}/#{file_name}")
      headers = { "Authorization" => "Bearer #{@access_token}" }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.head(uri.path, headers)
      raise APIError, res if res.code != "200"

      {
        "id" => file_id,
        "name" => file_name,
        "bytes" => res["content-length"],
        "md5" => res["content-md5"]
      }
    end

    def pre_upload(upload_contract, file_name, file_size, file_md5)
      fta = upload_contract["fta"]
      event = upload_contract["evento"]

      uri = URI.parse("#{base_uri}/gmt-sia-api/upload/#{fta}/#{event}/#{file_name}")
      headers = {
        "Authorization" => "Bearer #{@access_token}",
        "Content-MD5" => file_md5,
        "x-gmt-content-length" => file_size.to_s
      }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.head(uri.path, headers)
      raise APIError, res unless %w[204 206].include?(res.code)

      true
    end

    def pre_upload_from_path(upload_contract, path)
      file = File.open(path)
      file_name = File.basename(file.path)
      file_size = file.stat.size
      file_content = file.read
      file_md5 = Digest::MD5.hexdigest(file_content)

      pre_upload(upload_contract, file_name, file_size, file_md5)
    end

    def upload(upload_contract, file_name, file_size, file_md5, file_content)
      fta = upload_contract["fta"]
      event = upload_contract["evento"]

      uri = URI.parse("#{base_uri}/gmt-sia-api/upload/#{fta}/#{event}/#{file_name}")
      headers = {
        "Authorization" => "Bearer #{@access_token}", "Content-Type" => "application/octet-stream",
        "Content-MD5" => file_md5, "Content-Length" => file_size.to_s
      }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.put(uri.path, file_content, headers)
      raise APIError, res unless %w[200 201].include?(res.code)

      JSON.parse(res.body)
    end

    def upload_from_path(upload_contract, path)
      file = File.open(path)
      file_name = File.basename(file.path)
      file_size = file.stat.size
      file_content = file.read
      file_md5 = Digest::MD5.hexdigest(file_content)

      upload(upload_contract, file_name, file_size, file_md5, file_content)
    end

    def download(available_download_file)
      file_id = available_download_file["id"]
      file_name = available_download_file["nome"]

      uri = URI.parse("#{base_uri}/gmt-sia-api/download/#{file_id}/#{file_name}")
      headers = { "Authorization" => "Bearer #{@access_token}" }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.get(uri.path, headers)
      raise APIError, res if res.code != "200"

      res.body
    end

    def download_to_path(available_download_file, path)
      file_name = available_download_file["nome"]
      file_content = download(available_download_file)

      File.write("#{path}/#{file_name}", file_content)

      available_download_file["file_path"] = "#{path}/#{file_name}"
      available_download_file
    end

    def query_protocols(params = {})
      uri = URI.parse("#{base_uri}/gmt-protocolo-api/listaProtocolos")
      headers = { "Authorization" => "Bearer #{@access_token}", "Content-Type" => "application/json" }

      http = Net::HTTP.start(uri.host, uri.port, use_ssl: true)
      res = http.post(uri.path, params.to_json, headers)
      raise APIError, res if res.code != "200"

      JSON.parse(res.body)
    end
  end
end
